#include <petscdmda.h>
#include <petscksp.h>
//#include <petscpcmg.h>
#include <mpi.h>
#include "ComputeFunctions.h"
#include "def.h"
// -- 0.0 --  "ComputerFunctions" Global Functions

#undef __FUNCT__
#define __FUNCT__ "SetNonUniformGrid"

PetscErrorCode SetNonUniformGrid(DM da) {
  DM             cda;
  Vec            coords;
  DMDACoor3d       ***_coords;
  PetscInt       si,sj,sk,nx,ny,nz;
  PetscErrorCode ierr;
  int i, j, k;

  PetscFunctionBeginUser;

  ierr = DMGetCoordinateDM(da,&cda);CHKERRQ(ierr);
  ierr = DMGetCoordinatesLocal(da,&coords);CHKERRQ(ierr);
  ierr = DMDAVecGetArray(cda,coords,&_coords);CHKERRQ(ierr);
  ierr = DMDAGetGhostCorners(cda,&si,&sj,&sk,&nx,&ny,&nz);CHKERRQ(ierr);
  printf("%d  %d  %d  %d  %d  %d \n", si, sj, sk, nx, ny, nz);
  for (k = sk; k < sk+nz-1; k++) {
    for (j = sj; j < sj+ny-1; j++) {
      for (i = si; i < si+nx-1; i++) {
        PetscRealPart(_coords[k][j][i].x) = si*dxValue + (i-si)*dxValue;
        
        PetscRealPart(_coords[k][j][i].y) = sj*dyValue + (j-sj)*dyValue*2.0;
        
        PetscRealPart(_coords[k][j][i].z) = sk*dzValue + (k-sk)*dzValue;
      }
    }
  }


  ierr = DMDAVecRestoreArray(cda,coords,&_coords);CHKERRQ(ierr);

  PetscFunctionReturn(0);

}

#undef __FUNCT__
#define __FUNCT__ "PlotGrid"

PetscErrorCode PlotGrid(DM da) {
  FILE *fpcoors;
  DMDACoor3d    ***coors;
  char fname[20];
  int i,j,k, xs, ys, zs, xw, yw, zw;
  DM cda;
  Vec gc;
 
  PetscErrorCode ierr;
  sprintf(fname, "%s%d%s", "Results/Coord", 0, ".xyz");
  fpcoors = fopen(fname, "w");

  ierr = DMDAGetCorners(da,&xs,&ys,&zs,&xw,&yw,&zw);CHKERRQ(ierr);
  ierr = DMGetCoordinateDM(da,&cda);
  ierr = DMGetCoordinatesLocal(da,&gc);
  ierr = DMDAVecGetArray(cda, gc, &coors);


  fprintf(fpcoors, "%d  %d  %d  \n", xw, yw, zw);

  for(k = zs; k < zs+zw; k++) {
    for(j = ys; j < ys+yw; j++) {
      for(i = xs; i < xs+xw; i++) {
	fprintf(fpcoors, "%f  ", coors[k][j][i].x);
      }
      fprintf(fpcoors, "\n");
    }
    fprintf(fpcoors, "\n");
  }

  for(k = zs; k < zs+zw; k++) {
    for(j = ys; j < ys+yw; j++) {
      for(i = xs; i < xs+xw; i++) {
	fprintf(fpcoors, "%f  ", coors[k][j][i].y);
      }
      fprintf(fpcoors, "\n");
    }
    fprintf(fpcoors, "\n");
  }

  for(k = zs; k < zs+zw; k++) {
    for(j = ys; j < ys+yw; j++) {
      for(i = xs; i < xs+xw; i++) {
	fprintf(fpcoors, "%f  ", coors[k][j][i].z);
      }
      fprintf(fpcoors, "\n");
    }
    fprintf(fpcoors, "\n");
  }

  fclose(fpcoors);

  ierr = DMDAVecRestoreArray(cda, gc, &coors);

}
