static char help[] = "Solves 2D inhomogeneous Laplacian using multigrid.\n\n";

#include <petscdmda.h>
#include <petscksp.h>
//#include <petscpcmg.h>
#include <petsc.h>
#include "ComputeFunctions.h"
#include "def.h"

extern PetscErrorCode PlotGrid(DM);
extern PetscErrorCode SetNonUniformGrid(DM);


#undef __FUNCT__
#define __FUNCT__ "main"
int main(int argc,char **argv) {
  FILE *fpCp;
  int rank, NumRank;
  DM             da;
  PetscReal      norm;
  PetscErrorCode ierr;
  PetscInt       stages[7];
  KSP	ksp;
  Vec	x,b,r;
  Mat	A;

  PetscInitialize(&argc,&argv,(char *)0,help);
  MPI_Comm_rank(PETSC_COMM_WORLD, &rank);
  MPI_Comm_size(PETSC_COMM_WORLD, &NumRank);
  if(!rank) { printf("NumRank = %d \n", NumRank);} else{}

  ierr = PetscLogStageRegister("DMMG Setup",&stages[0]);  CHKERRQ(ierr);
  ierr = PetscLogStageRegister("Pressure RHS Setup",&stages[1]);  CHKERRQ(ierr);
  ierr = PetscLogStageRegister("Pressure Solve",&stages[2]);  CHKERRQ(ierr);
  ierr = PetscLogStageRegister("Corrector RHS Setup",&stages[3]);  CHKERRQ(ierr);
  ierr = PetscLogStageRegister("Corrector Solve",&stages[4]);  CHKERRQ(ierr);

  ierr = PetscLogStagePush(stages[0]);CHKERRQ(ierr);   // Start DMMG Setup
  printf("Check1\n");
  ierr = KSPCreate(PETSC_COMM_WORLD,&ksp);CHKERRQ(ierr);
  ierr = DMDACreate3d(PETSC_COMM_WORLD,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_BOUNDARY_NONE,DMDA_STENCIL_STAR,-imax,-jmax,-kmax,PETSC_DECIDE,PETSC_DECIDE,PETSC_DECIDE,1,1,0,0,0,&da);CHKERRQ(ierr);
  printf("Check2\n");
  ierr = DMDASetUniformCoordinates(da, 0., (imax-1)*dxValue, 0., (jmax-1)*dyValue, 0, (kmax-1)*dzValue);CHKERRQ(ierr);
  ierr = SetNonUniformGrid(da);
  printf("Check3\n");


  ierr = PetscLogStagePop();  CHKERRQ(ierr); // Finish DMMG Setup 
  printf("Check4\n");
  ierr = PlotGrid(da);
  printf("Check5\n");
  ierr = KSPDestroy(&ksp);CHKERRQ(ierr);
  ierr = DMDestroy(&da);CHKERRQ(ierr);
  printf("Check7\n");
  ierr = PetscFinalize();



  return 0;
}


